var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
import { createType } from '../helpers';
import { asyncAction, createResetAction, syncAction } from './actions';
import { createReducer } from './reducer';
import { createSelectors } from './selectors';
var prefix = 'Redux_Tiles_';
export function createTile(params) {
    var type = params.type, fn = params.fn, caching = params.caching, nesting = params.nesting, _a = params.selectorFallback, selectorFallback = _a === void 0 ? null : _a;
    // initial state is equal to empty object, because of possible nesting
    // basically every object should contain default properties, so we handle
    // this situation using selectors
    var initialState = nesting ? {} : null;
    var identificator = createType({ type: type });
    var types = {
        START: "" + prefix + identificator + "_START",
        SUCCESS: "" + prefix + identificator + "_SUCCESS",
        FAILURE: "" + prefix + identificator + "_FAILURE",
        RESET: "" + prefix + identificator + "_RESET"
    };
    var selectorParams = {
        selectorFallback: {
            isPending: false,
            error: null,
            data: selectorFallback,
            fetched: false,
        },
        tileName: type,
        nesting: nesting
    };
    var selectors = createSelectors(selectorParams);
    var actionParams = {
        START: types.START,
        SUCCESS: types.SUCCESS,
        FAILURE: types.FAILURE,
        fn: fn,
        type: type,
        caching: caching,
        nesting: nesting,
        selectors: selectors
    };
    var action = asyncAction(actionParams);
    action.reset = createResetAction({ type: types.RESET });
    var reducerObject = (_b = {},
        _b[types.START] = {
            data: null,
            isPending: true,
            error: null,
            fetched: false,
        },
        _b[types.FAILURE] = function (_storeState, storeAction) { return ({
            data: null,
            isPending: false,
            error: storeAction.error,
            fetched: true,
        }); },
        _b[types.SUCCESS] = function (_storeState, storeAction) { return ({
            error: null,
            isPending: false,
            data: storeAction.payload && storeAction.payload.data,
            fetched: true,
        }); },
        _b[types.RESET] = initialState,
        _b);
    var reducer = createReducer(initialState, reducerObject);
    return { action: action, reducer: reducer, selectors: selectors, tileName: type, constants: types, reflect: params };
    var _b;
}
export function createSyncTile(params) {
    var type = params.type, nesting = params.nesting, _a = params.fn, fn = _a === void 0 ? function (fnParams) { return fnParams.params; } : _a, fns = params.fns, 
    // we have default state as an object because of the possible nesting
    _b = params.initialState, 
    // we have default state as an object because of the possible nesting
    initialState = _b === void 0 ? nesting ? {} : null : _b, selectorFallback = params.selectorFallback;
    var identificator = createType({ type: type });
    var types = {
        SET: "" + prefix + identificator + "_SET",
        RESET: "" + prefix + identificator + "_RESET"
    };
    var selectorParams = {
        selectorFallback: selectorFallback,
        tileName: type,
        nesting: nesting
    };
    var selectors = createSelectors(selectorParams);
    var actionParams = {
        SET: types.SET,
        nesting: nesting,
        fn: fn
    };
    var action = syncAction(actionParams);
    action.reset = createResetAction({ type: types.RESET });
    if (fns) {
        Object.keys(fns).forEach(function (methodName) {
            var method = fns[methodName];
            var customActionParams = __assign({}, actionParams, { fn: method });
            action[methodName] = syncAction(customActionParams);
        });
    }
    var reducerObject = (_c = {},
        _c[types.SET] = function (_storeState, storeAction) {
            return storeAction.payload && storeAction.payload.data;
        },
        _c[types.RESET] = initialState,
        _c);
    var reducer = createReducer(initialState, reducerObject);
    return { action: action, selectors: selectors, reducer: reducer, tileName: type, constants: types, reflect: params };
    var _c;
}
//# sourceMappingURL=index.js.map